// ------- prelude for SD card section
# include <SD.h> // library for writing and reading from the SD card

// don't need these definitions, just useful to make sure they're 
//claimed as SD card stuff and not forgotten about later
const int MOSI_reminder = 11; // to DI
const int MISO_reminder = 12; // to DO
const int CLK_reminder = 13; // to CLK

// chip select pin can be anything
const int CS = 4; 

// status LEDs to indicate if everything is going ok
const int status_red = 5;
const int status_green = 6;

// minimum number of bytes for an 8.3 filename
const int filename_bytes = 12;

// useful to have it as a global variable
char* filename;

Sd2Card card;

//---------- servo and sensor

#include <Servo.h>
#include <Adafruit_Sensor.h>
#include <Adafruit_MPU6050.h>

Servo pitch;
Servo roll;

const int pitch_pin = 10;
const int roll_pin = 9;

Adafruit_MPU6050 mpu;


void setup() {
  Serial.begin(9600);
  Serial.println("serial is working");

  // get status LEDs set up
  pinMode(status_red, OUTPUT);
  pinMode(status_green, OUTPUT);
  
  // initialise the card on the defined pin
  initCard(CS);

  // initialise the mpu
  mpu_init();

  // setup servos
  setup_servos();
  
}

void loop() {
  /* Get new sensor events with the readings */
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);
  
  float x_raw = a.acceleration.x;
  float y_raw = a.acceleration.y;
  float z_raw = a.acceleration.z;
  
  float total_acceleration = sqrt(sq(x_raw) + sq(y_raw) + sq(z_raw));


  float x_norm = x_raw / total_acceleration;
  float y_norm = y_raw / total_acceleration;
  float z_norm = z_raw / total_acceleration;

  int pitch_out = lerp(x_norm, -1, 1, 0, 180);

  pitch.write(pitch_out);
  roll.write(lerp(y_norm, -1, 1, 0, 180));
  delay(20);
}

void write_to_card(char* file, char* toWrite) {
  /*
   * natalie's responsibility
   * 
   * takes in a filename and writes to it, and closes the file afterwards 
   */
  Serial.print("writing to ");
  Serial.println(file);
  int now = millis();
  File location = SD.open(file, FILE_WRITE);
  location.print(now);
  location.print(": ");
  location.println(toWrite);
  location.close();
}

bool initCard(int pin) {
  /*
   * natalie's responsibility
   * 
   * tries to intialise the SD card and get the filename to use
   */ 

  Serial.println("initialising SD card");

  digitalWrite(status_red, true); // turn red LED on
  
  SD.begin(pin);
  
  // can the card be initialised?
  if (!card.init(SPI_HALF_SPEED, CS)) {
    return false;
  } else {
     Serial.println("couldn't initialise SD card, exiting");
     while (true) {
      delay(1);
     }
  }

  filename = get_filename();

  digitalWrite(status_red, 0);

  Serial.print("activating the SD card worked, filename of ");
  Serial.println(filename);
  
  return true;
}

char* get_filename() {
  /*
   * function to get the filename to use, so each test gets a unique 
   * file
   * 
   */
  int i = 0;

  // files will follow the pattern test(number).csv
  char base_name[] = "test";
  char ending[] = ".csv";

  // need to allocate so the filename can be stored.
  // won't free this variable as it should always be useful
  char* return_memory = malloc(filename_bytes * sizeof(char));
  
  while (true) {
    // this checks every loop to see if a filename is free. 
    // if so, it returns it
    // if not, it tries the next one
    
    char filename [filename_bytes];
    sprintf(filename, "%s%d%s", base_name, i, ending); // formats the filename
    if (SD.exists(filename)) {
      i++;
    }else{
      strcpy(return_memory, filename);
      return return_memory;
    }
  }
}

float lerp(float x, float x_0, float x_1, float y_0, float y_1) {
  float first = y_0 * (x_1 - x);
  float second = y_1 * (x - x_0);
  float denominator = x_1 - x_0;

  return (first + second) / denominator;
}

void mpu_init() {
  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  
  Serial.println("started MPU");
  mpu.setAccelerometerRange(MPU6050_RANGE_8_G);
  mpu.setGyroRange(MPU6050_RANGE_500_DEG);
  mpu.setFilterBandwidth(MPU6050_BAND_21_HZ);
}

void setup_servos() {
  pitch.attach(pitch_pin);
  roll.attach(roll_pin);

  pitch.write(0);
  roll.write(0);
}
